class DualWeapon extends DeusExWeapon;


const A_FIRE=1;
const A_RELOAD=2;
const A_CYCLEAMMO=3;
const A_UP=4;
const A_DOWN=5;


var() travel bool bDual; //if true, owner is currently in dual mode

var DualWeapon Second;

var byte InventoryGroup2;

var Mesh AltPlayerViewMesh,
         Alt3rdViewMesh;

var float ActionTimer,ActionValue;
var int PendingAction;

//=============================================================================
// PreBeginPlay

function PreBeginPlay()
{
Super.PreBeginPlay();

if(AltPlayerViewMesh==None)
	AltPlayerViewMesh=PlayerViewMesh;
if(Alt3rdViewMesh==None)
	Alt3rdViewMesh=ThirdPersonMesh;
}


//=============================================================================
// Functions that return values relating to the second weapon

function DualWeapon SecondWeapon()
{
local DualWeapon w;

if(owner==None)
	return None;

foreach owner.ChildActors(class'DualWeapon',w)
	if(w!=self && w.class==class)
		return w;

return None;
}

function bool bCanDual()
{
local DualWeapon w;

foreach owner.ChildActors(class'DualWeapon',w)
	if(w!=self && w.class==class)
		return True;

return False;
}

function bool bPrimary()
{
local DualWeapon w;
w=SecondWeapon();

return (w!=None && bCanDual() && bDual && !w.bDual);
}

function bool bSecondary()
{
local DualWeapon w;
w=SecondWeapon();

return (w!=None && bCanDual() && !bDual && w.bDual);
}

function bool bSolo()
{
local DualWeapon w;
w=SecondWeapon();

return (!bDual && (!w.bDual || w==None));
}
//=============================================================================
// The almighty TICK

function Tick(float deltaTime)
{
Super.Tick(deltaTime);

if(ActionTimer>0.0)
	{
	ActionTimer-=deltaTime;
	if(ActionTimer<=0.0)
		{
		HandlePendingAction();
		}
	}

if(bPrimary() || bSolo())
	{
	PlayerViewOffset.y=default.PlayerViewOffset.y*-100;
	FireOffset.y=(-default.FireOffset.y);
	PlayerViewMesh=AltPlayerViewMesh;
	ThirdPersonMesh=Alt3rdViewMesh;
	}
else
	{
	PlayerViewOffset.y=default.PlayerViewOffset.y*100;
	FireOffset.y=default.FireOffset.y;
	PlayerViewMesh=default.PlayerViewMesh;
	ThirdPersonMesh=default.ThirdPersonMesh;
	}
}

//=============================================================================
// ToggleDual() - actually takes out the second weapon

exec function DualWeapon()
{
ToggleDual();
}

exec function ToggleDual()
{
if(!bCanDual()) return;

Second=SecondWeapon();

bDual=!bDual;

if(bDual) //turn on dual mode
	{
	DeusExPlayer(owner).ClientMessage(ItemName$" in dual mode.");
	Second.bDual=False; //secondary weapon thinks it's in single mode
	Second.GotoState('Idle2');
	Second.bHidden=False;
	Second.PlaySelect();
	}
else
	{
	DeusExPlayer(owner).ClientMessage(ItemName$" in single mode.");
	Second.PutDown();
	Second=None;
	}
}


//=============================================================================
// If we only have one, give us another!


function bool HandlePickupQuery(Inventory Item)
{
local DualWeapon W;
	
// make sure that if you pick up a modded weapon that you
// already have, you get the mods
W = DualWeapon(Item);
if ((W != None) && (W.Class == Class))
	{
	if (W.ModBaseAccuracy > ModBaseAccuracy)
		ModBaseAccuracy = W.ModBaseAccuracy;
	if (W.ModReloadCount > ModReloadCount)
		ModReloadCount = W.ModReloadCount;
	if (W.ModAccurateRange > ModAccurateRange)
		ModAccurateRange = W.ModAccurateRange;

	// these are negative
	if (W.ModReloadTime < ModReloadTime)
		ModReloadTime = W.ModReloadTime;
	if (W.ModRecoilStrength < ModRecoilStrength)
		ModRecoilStrength = W.ModRecoilStrength;

	if (W.bHasLaser)
		bHasLaser = True;
	if (W.bHasSilencer)
		bHasSilencer = True;
	if (W.bHasScope)
		bHasScope = True;

	// copy the actual stats as well
	if (W.ReloadCount > ReloadCount)
		ReloadCount = W.ReloadCount;
	if (W.AccurateRange > AccurateRange)
		AccurateRange = W.AccurateRange;

	// these are negative
	if (W.BaseAccuracy < BaseAccuracy)
		BaseAccuracy = W.BaseAccuracy;
	if (W.ReloadTime < ReloadTime)
		ReloadTime = W.ReloadTime;
	if (W.RecoilStrength < RecoilStrength)
		RecoilStrength = W.RecoilStrength;

	if(!bCanDual()) //if we only have one, GIVE US ANOTHER!!!!!
		{
		W.InventoryGroup=default.InventoryGroup;
		InventoryGroup=InventoryGroup2;

		return False;
		}
	}

return Super.HandlePickupQuery(item);
}



//=============================================================================
// Standard functions that have been overridden to call on secondary weapon if
// necessary

function bool PutDown()
{
Super.PutDown();

if(Second!=None && bDual)
	Second.SetPendingAction(A_DOWN,0.8);
}

function Fire(float Value)
{
Super.Fire(value);

if(Second!=None && bDual)
	{
	Second.SetPendingAction(A_FIRE,ShotTime/2,value);
	}
}

function CycleAmmo()
{
Super.CycleAmmo();

if(Second!=None && bDual)
	Second.SetPendingAction(A_CYCLEAMMO,0.8);
}

function ReloadAmmo()
{
Super.ReloadAmmo();

if(Second!=None && bDual)
	Second.SetPendingAction(A_RELOAD,0.8);
}

function PlaySelect()
{
Super.PlaySelect();

if(Second!=None && bDual)
	Second.SetPendingAction(A_UP,0.8);
}


simulated event RenderOverlays(canvas Canvas)
{
Super.RenderOverlays(Canvas);

if(Second!=None && bDual)
	Second.RenderOverlays(Canvas);
}

//=============================================================================
// Called on secondary weapon to fire after a short delay

function SetPendingAction(int action,float timer,optional float value)
{
//if we already have an action, deal with it early and move on
if(ActionTimer>0.0)
	HandlePendingAction();

PendingAction=action;
ActionValue=value;
ActionTimer=timer;
}

//=============================================================================
// Handle action

function HandlePendingAction()
{
switch(PendingAction)
	{
	case A_FIRE:		Fire(ActionValue); break;
	case A_RELOAD:		ReloadAmmo(); break;
	case A_CYCLEAMMO:	CycleAmmo(); break;
	case A_UP:		GotoState('Idle2'); bHidden=False; PlaySelect(); break;
	case A_DOWN:		PutDown(); break;
	}
}


//=============================================================================

defaultproperties
{
bDual=False
ActionTimer=0.0
}