class NewPedestrianGenerator extends Actor;

var(Pedestrian) int NumPedestrians; //max 32
var(Pedestrian) class<ScriptedPawn> PedClasses[8]; //classes to spawn, chosen randomly

var(SpawnLocation) bool bOutOfLOS; //if true, only spawn pedestrians when out of player's LOS
var(SpawnLocation) name AltSpawnTag; //if bOutOfLOS is true, and the generator is in player's LOS, use alternate SpawnPoints with this tag

var(Pedestrian) name PedOrders,PedOrdersTag;

var(Pedestrian) string PedBindName; //if not blank, will set the pedestrians' bind name to this

var ScriptedPawn pedestrians[32];
var int numClasses; //number of classes

function PostBeginPlay()
{
Super.PostBeginPlay();

if(PedClasses[0]==None)
	{
	Destroy();
	return;
	}

if(NumPedestrians>32) NumPedestrians=32;

for(numClasses=0;(numClasses<8 && PedClasses[numClasses]!=None);numClasses++)
	{}

SetTimer(1.0,True);
}

function Timer()
{
if(GetNumPds() < NumPedestrians)
	SpawnPed();
}

function int GetNumPds() //returns the number of pedestrians
{
local int x,y;

y=0;

for(x=0;x<32;x++)
	{
	if(pedestrians[x]!=None)
		y++;
	}

return y;
}

function SpawnPed()
{
local int x;

for(x=0;(x<32 && pedestrians[x]!=None);x++)
	{}
if(x>=32) return; //no space for more peds

if(EstablishSpawnLoc()==vect(0,0,0)) //can't spawn at this time!!
	return;

pedestrians[x]=Spawn(PedClasses[Rand(numClasses+1)],self,,EstablishSpawnLoc());

if(pedestrians[x]==None)
	pedestrians[x]=Spawn(PedClasses[Rand(numClasses+1)],self,,EstablishSpawnLoc(True));

if(pedestrians[x]!=None && PedBindName!="")
	pedestrians[x].BindName=PedBindName;
if(pedestrians[x]!=None && PedOrders!='None')
	pedestrians[x].SetOrders(PedOrders,PedOrdersTag,True);
}

function Vector EstablishSpawnLoc(optional bool bUseAlt) //bUseAlt - if passed & true, use an alternate location
{
local SpawnPoint p;
local Vector v,v2;

if((bOutOfLOS && PlayerCanSeeMe()) || bUseAlt)
	{
	if(AltSpawnTag=='None')
		return Vect(0,0,0);

	foreach AllActors(class'SpawnPoint',p,AltSpawnTag)
		{
		if(p!=None)
			if( (!p.PlayerCanSeeMe()) || (bUseAlt && !bOutOfLOS))
				{
				v=p.Location;
				v-=(Vect(1,1,0) * p.CollisionRadius);
				v2=VRand(); v2.Z=0.0; v2=v2*2*p.CollisionRadius;
				return v+v2;
				}
		}
	}
else
	{
	v=Location;
	v-=(Vect(1,1,0) * CollisionRadius);
	v2=VRand(); v2.Z=0.0; v2=v2*2*CollisionRadius;
	return v+v2;
	}

return Vect(0,0,0);
}

defaultproperties
{
bHidden=True
NumPedestrians=8
PedClasses(0)=class'LowerClassMale'
PedClasses(1)=class'LowerClassMale2'
PedClasses(2)=class'LowerClassFemale'
bOutOfLOS=True
PedBindName=""
PedOrders=Wandering
CollisionRadius=64.0
}